const std = @import("std");

pub fn main() !u8 {
    const stderr = std.io.getStdErr().writer();

    var device_dir = try std.fs.cwd().openDir(
        "/sys/devices/pci0000:00/0000:00:15.1",
        .{},
    );
    defer device_dir.close();

    var tree: TreeDigger = .{};

    tree.digForDevice(
        device_dir,
        &.{ "i2c_designware.", "i2c-", "i2c-ELAN901C:", "0018:04F3:2C29" },
    ) catch |err| switch (err) {
        error.TreeNotFound => {
            try stderr.writeAll("somewhere we screwed up\n");
            return 1;
        },
        else => return err,
    };

    var unbind = try std.fs.cwd().openFile(
        "/sys/bus/hid/drivers/hid-multitouch/unbind",
        .{ .mode = .write_only },
    );
    defer unbind.close();

    unbind.writeAll(&tree.target) catch |err| switch (err) {
        error.Unexpected => {
            try stderr.print(
                "Couldn't unbind {s} (perhaps it is already unbound?)\n",
                .{&tree.target},
            );
            return 1;
        },
        else => return err,
    };

    try stderr.print("Successfully unbound {s}\n", .{&tree.target});

    return 0;
}

const DigError = error{
    TreeNotFound,
};

const TreeDigger = struct {
    target: [19]u8 = [_]u8{0} ** 19,

    pub fn digForDevice(self: *TreeDigger, parent: std.fs.Dir, layers: []const []const u8) !void {
        const stderr = std.io.getStdErr().writer();

        var walker = parent.iterate();
        walk: while (try walker.next()) |entry| {
            if (std.mem.startsWith(u8, entry.name, layers[0])) {
                try stderr.print("Found: {s}\n", .{entry.name});

                if (layers.len == 1) {
                    @memcpy(&self.target, entry.name);
                    return;
                } else {
                    var child = try parent.openDir(entry.name, .{});
                    defer child.close();

                    return if (self.digForDevice(child, layers[1..])) |_|
                        void{}
                    else |err| if (err == DigError.TreeNotFound)
                        continue :walk
                    else
                        err;
                }
            }
        }

        return DigError.TreeNotFound;
    }
};
